@@ -126,7 +126,7 @@ setMethod("trackNames", "QuickloadGenome", function(object) {
 setMethod("mcols", "QuickloadGenome", function(x) {
   files <- QuickloadGenome_annotFiles(x)
   if (!length(xmlChildren(files)))
-    new("DataFrame", nrows = length(x))
+    make_zero_col_DFrame(length(x))
   else
     Reduce(function(x, y) merge(as.data.frame(as.list(x)),
                                 as.data.frame(as.list(y)), all = TRUE),
@@ -153,15 +153,6 @@ setMethod("toString", "GenomeDescription", function(x) {
   paste(organism(x), provider(x), providerVersion(x))
 })
 
-## Not sure where this method should land. I'm sure it will be useful
-## for publishing adhoc genomes through Quickload.
-setMethod("seqinfo", "DNAStringSet", function(x) {
-  x_names <- names(x)
-  if (is.null(x_names))
-    x_names <- as.character(seq(length(x)))
-  Seqinfo(x_names, width(x))
-})
-
 QuickloadGenome <- function(quickload, genome, create = FALSE,
                             seqinfo = GenomicRanges::seqinfo(genome),
                             title = toString(genome))
