@@ -8,7 +8,7 @@ setClass("UCSCSession",
 
 # gets an 'hgsid' to initialize the session
 setMethod("initialize", "UCSCSession",
-          function(.Object, url = "http://genome.ucsc.edu/cgi-bin/",
+          function(.Object, url = "https://genome.ucsc.edu/cgi-bin/",
                    user = NULL, session = NULL, force = FALSE, ...)
           {
             .Object@url <- url
@@ -17,7 +17,8 @@ setMethod("initialize", "UCSCSession",
             gw <- httpGet(gwURL, cookiefile = tempfile(), header = TRUE,
                           .parse=FALSE, ...)
             if (grepl("redirectTd", gw)) {
-                url <- sub(".*?a href=\"h([^[:space:]]+cgi-bin/).*", "h\\1", gw)
+                url <- sub(".*?a href=\"http([^[:space:]]+cgi-bin/).*",
+                           "https\\1", gw)
                 return(initialize(.Object, url, user=user, session=session,
                                   force=TRUE, ...))
             }
@@ -261,7 +262,7 @@ normTableQueryRange <- function(range, genome, max.length = 1000L) {
 }
 
 ucscTables <- function(genome, track) {
-  url <- "http://genome.ucsc.edu/cgi-bin/hgTables"
+  url <- "https://genome.ucsc.edu/cgi-bin/hgTables"
   if (!isSingleString(genome))
     stop("'genome' must be a single non-NA string")
   # check genome is valid or not
@@ -288,19 +289,16 @@ setMethod("ucscTableQuery", "UCSCSession",
 normArgTrack <- function(name, trackids) {
   if (!isSingleString(name))
     stop("'track' must be a single string")
-  if (!(name %in% trackids)) {
-    mapped_name <- trackids[name]
-    if (is.na(mapped_name))
+  if (name %in% names(trackids))
+      unname(trackids[name])
+  else
       stop("Unknown track: ", name)
-    name <- mapped_name
-  }
-  unname(name)
 }
 
 setMethod("ucscTableQuery", "character",
           function(x, track = NULL, range =  NULL, table = NULL,
                    names = NULL, intersectTrack = NULL, check = TRUE, hubUrl = NULL,
-                   genome = NULL, url = "http://genome.ucsc.edu/cgi-bin/") {
+                   genome = NULL, url = "https://genome.ucsc.edu/cgi-bin/") {
               stopifnot(isSingleString(x))
               if (!is.null(intersectTrack))
                 stop("intersectTrack is no longer supported")
@@ -360,7 +358,7 @@ setReplaceMethod("hubUrl", "UCSCTableQuery", function(x, value) {
 
 ## gets the track names available from the table browser
 ucscTableTracks <- function(genome) {
-  doc <- httpGet("http://genome-euro.ucsc.edu/cgi-bin/hgTables", c(db = genome, hgta_group = "allTracks"))
+  doc <- httpGet("https://genome.ucsc.edu/cgi-bin/hgTables", c(db = genome, hgta_group = "allTracks"))
   label_path <- "//select[@name = 'hgta_track']/option/text()"
   labels <- sub("\n.*$", "", sapply(getNodeSet(doc, label_path), xmlValue))
   track_path <- "//select[@name = 'hgta_track']/option/@value"
@@ -487,9 +485,14 @@ setMethod("track", "UCSCTableQuery",
                 cbind(table,strand)
               }
 
+              if (is.null(table[["chrom"]]))
+                  table[["chrom"]] <- as.character(seqnames(object@range))
+
               # create GRange object
-              output <- GRanges(seqnames = table[["chrom"]], ranges = IRanges(start = table[["start"]],
-                                  end = table[["end"]]), strand = table[["strand"]])
+              output <- GRanges(seqnames = table[["chrom"]],
+                                ranges = IRanges(start = as.numeric(table[["start"]]),
+                                                 end = as.numeric(table[["end"]])),
+                                                 strand = table[["strand"]])
 
               # remove used columns
               table[["chrom"]] <- table[["strand"]] <- table[["start"]] <- table[["end"]] <- NULL
@@ -516,8 +519,11 @@ setMethod("track", "UCSCTableQuery",
 ##             set
 ##           })
 
-parseResponse <- function(response, tableName) {
+parseResponse <- function(response, tableName, chrom) {
   results <- response[[tableName]]
+  if (is.null(results)) {
+    return(as.data.frame(do.call(rbind, response[[chrom]])))
+  }
   if (is.null(names(results))) {
     df <- do.call(rbind.data.frame, results)
   } else {
@@ -559,7 +565,8 @@ setMethod("getTable", "UCSCTableQuery",
             } else {
               url <- RestUri(paste0(object@url, "hubApi"))
               response <- read(url$getData$track, query)
-              output <- parseResponse(response, tableName)
+              seqnames <- as.character(seqnames(object@range))
+              output <- parseResponse(response, tableName, seqnames)
               NAMES <- names(object)
               if (!is.null(NAMES)) { # filter by NAMES
                 if (is.null(output$name))
@@ -1415,6 +1422,7 @@ setMethod("ucscCart", "UCSCSession",
             mat <- matrix(unlist(pairs), nrow = 2)
             vals <- mat[2,]
             names(vals) <- mat[1,]
+            vals[["db"]] <- sub(".*_", "", vals[["db"]])
             new("ucscCart", vals)
           })
 setMethod("ucscCart", "UCSCView",
@@ -1451,7 +1459,7 @@ setMethod("ucscTracks", "UCSCSession",
             nodes <- getNodeSet(tracks, "//select/@name")
             trackIds <- unlist(nodes)
             ##trackIds <- sapply(nodes, xmlValue)
-            nodes <- getNodeSet(tracks, "//select/../a[not(@class)]/text()")
+            nodes <- getNodeSet(tracks, "//select/../a[not(@class='pennantIconText')]/text()")
             nms <- sapply(nodes, xmlValue)
             names(trackIds) <- sub("^ ", "", nms)
             new("ucscTracks", ids = trackIds, modes = trackModes)
@@ -1475,7 +1483,7 @@ setMethod("ucscTrackModes", "ucscTracks",
 ucscGenomes <- function(organism=FALSE) {
   stopifnot(isTRUEorFALSE(organism))
   names <- c("db", "species", "date", "name", "organism")
-  url <- RestUri("http://api.genome.ucsc.edu/")
+  url <- RestUri("https://api.genome.ucsc.edu/")
   response <- read(url$list$ucscGenomes)
   genomes <- response[[5]]
   genomeNames <- names(genomes)
