@@ -7,6 +7,7 @@
 #include "ucsc/bwgInternal.h"
 
 #include "bigWig.h"
+#include "bbiHelper.h"
 #include "handlers.h"
 
 static struct bwgBedGraphItem *
@@ -212,23 +213,10 @@ SEXP BWGSectionList_cleanup(SEXP r_sections)
 /* --- .Call ENTRY POINT --- */
 SEXP BWGFile_seqlengths(SEXP r_filename) {
   pushRHandlers();
+  SEXP seqlengths;
   struct bbiFile * file = bigWigFileOpen((char *)CHAR(asChar(r_filename)));
-  struct bbiChromInfo *chromList = bbiChromList(file);
-  struct bbiChromInfo *chrom = chromList;
-  SEXP seqlengths, seqlengthNames;
-  
-  PROTECT(seqlengths = allocVector(INTSXP, slCount(chromList)));
-  seqlengthNames = allocVector(STRSXP, length(seqlengths));
-  setAttrib(seqlengths, R_NamesSymbol, seqlengthNames);
-  
-  for(int i = 0; i < length(seqlengths); i++) {
-    INTEGER(seqlengths)[i] = chrom->size;
-    SET_STRING_ELT(seqlengthNames, i, mkChar(chrom->name));
-    chrom = chrom->next;
-  }
-  
+  PROTECT(seqlengths = bbiSeqLengths(file));
   bbiFileClose(&file); 
-  bbiChromInfoFreeList(&chromList);
   popRHandlers();
   UNPROTECT(1);
   return seqlengths;
